#include <EngduinoSD.h>
#include <EngduinoLight.h>
#include <EngduinoThermistor.h>
#include <stdlib.h>
#include <stdbool.h>
#include <SPI.h>

const char* fileName = "DATA.txt";
boolean openFile(void);
boolean closeFile(void); // where does this go ?!
boolean writeToFile(String str); // write to file already opened
boolean beginDevices(void);

String compressData(char* sensor, String data);
boolean sendData(char* sensor, String data);
boolean sendDataToSD(String dataToSend);
boolean sendDataToSerial(String dataToSend);

boolean processData();

void setup(){
  if(!beginDevices()) {
    Serial.println("E Did not successfully initialize devices.");
    return;
  }
}

void loop() {
  if(!processData()) {
    closeFile();
    return;
  }
}

boolean beginDevices() {
  delay(1000); // Delay to make sure there is no corruption
  boolean result = true;
  EngduinoLight.begin();
  EngduinoThermistor.begin(); 
  Serial.begin(115200); 
  result = result && EngduinoSD.begin();
  result = result && openFile();
  return result;
}

boolean openFile() {
  boolean didOpen = EngduinoSD.open(fileName, FILE_WRITE);
  if(!didOpen)
    Serial.println("E Did not open." + String(fileName));
  return didOpen;
}

boolean closeFile() {
  boolean didClose = EngduinoSD.close();
  if(!didClose)
    Serial.println("E Did not close." + String(fileName));
  return didClose;
}

boolean writeToFile(const char* fileName, String message){
  boolean didWrite = EngduinoSD.write(message + "\r\n");
  if(!didWrite)
    Serial.println("E Did not successfully write \"" + message + "\" to file \"" + String(fileName) + "\".");
  return didWrite;
}

String compressData(char* sensor, String data){
  char c = sensor[0];
  String compress = String(c) + " " + data;
  return compress;
}

boolean sendDataToSerial(String dataToSend) {
  boolean didSend = Serial.println(dataToSend);
  if(!didSend)
    Serial.println("E Did not successfully send data \"" + dataToSend + "\" over the serial.");
  return didSend;
}

boolean sendDataToSD(String dataToSend) {
  return writeToFile("/Engduino/DATA.txt", dataToSend);
}

boolean sendData(char* sensor, String data) {
  String compressedData = compressData(sensor, data);
  boolean sendSD = sendDataToSD(compressedData);
  boolean sendSerial = sendDataToSerial(compressedData);
  return sendSD && sendSerial;
}
/*
boolean processData() {
  if (Serial.readString() != "S"){
    /*String gps = String(analogRead(0)); // E.g.
    sendData("GPS", gps);
    String humidity = "";
    sendData("Humidity", humidity);
    String wind = "";
    sendData("Wind", wind);
    String magnetic = "";
    sendData("Magnetic", magnetic);
    String pressure = "";
    sendData("Pressure", pressure);
    String noise = "";
    sendData("Noise", noise);
    uint16_t l = EngduinoLight.lightLevel();  
    String light = String(l);
    sendData("Light", light);
    float t = EngduinoThermistor.temperature();
    String temp = String(t);
    sendData("Temperature", temp); 
    return true;
  }
  
  // Stop
  Serial.println("S");
  return false;
}*/

boolean processData() {
/*String gps = String(analogRead(0)); // E.g.
  sendData("GPS", gps);
  String humidity = "";
  sendData("Humidity", humidity);
  String wind = "";
  sendData("Wind", wind);
  String magnetic = "";
  sendData("Magnetic", magnetic);
  String pressure = "";
  sendData("Pressure", pressure);
  String noise = "";
  sendData("Noise", noise);*/
  uint16_t l = EngduinoLight.lightLevel();  
  String light = String(l);
  sendData("Light", light);
  //delay(1000);
  float t = EngduinoThermistor.temperature();
  String temp = String(t);
  sendData("Temperature", temp); 
  //delay(1000);
  return true;
}


